 /**********************************************************************
 *
 *
 *		    ROVER object execution engine
 *			  Anthony D. Joseph
 *		   Laboratory for Computer Science
 *		Massachusetts Institute of Technology
 *
 * Module: HTRoverFile.c
 *
 * Description: Common server and client file functions
 *
 *
 ***********************************************************************
 *   Copyright 1997
 *   Anthony D. Joseph
 *   Massachusetts Institute of Technology
 *
 *   Permission to use, copy, modify, and distribute this program
 *   for any purpose and without fee is hereby granted, provided
 *   that this copyright and permission notice appear on all copies
 *   and supporting documentation, the name of M.I.T. not be used
 *   in advertising or publicity pertaining to distribution of the
 *   program without specific prior permission, and notice be given
 *   in supporting documentation that copying and distribution is
 *   by permission of M.I.T.  M.I.T. makes no representations about
 *   the suitability of this software for any purpose.  It is pro-
 *   vided "as is" without express or implied warranty.		
 ***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/timeb.h>
#include <strings.h>
#include "RoverBase.h"
#include "RoverFile.h"

/* This function is similar to mkstemp, except that it will avoid the letter
   shortage problem. Also it sets the close on exec flag for the file.
   It takes a template with EIGHT trailing Xs instead of SIX.
 */
PUBLIC FILE *RoverGetUniqFile(char *temp)
{
  int pid, len, i, fd, j;
  struct timeb now;
  char *ptr, *c;
  FILE *file;

  if ((!temp) || ((len = strlen(temp)) < 8)) return(NULL);
  ptr = c = temp+len-8;
  for (i = 0; i < 7; i++, c++)
    if (*c != 'X') return(NULL);

  /* We make a unique name by taking the process id and the three bytes
     representing the current millisecond interval. We iterate over the
     interval and pid, in case of a clash.
   */

  pid = getpid();
  for (j = pid+3; pid < j; pid++)
    for (i = 10; i >= 0; i--) {
      ftime(&now);
      sprintf(ptr, "%03X%05d",
	      (unsigned int)(((now.time*1000)+now.millitm)&0xFFF), pid);
      fd = open(temp,O_CREAT|O_EXCL|O_WRONLY|O_TRUNC, 0660);
      if (fd != -1) {
	close(fd);				/* Close original */
	file = fopen(temp, "w");		/* Open as file handle */
	fcntl(fileno(file), F_SETFD, 1);	/* Set to close on exec */
	return(file);
      }
    }
  return(NULL);
}

