#ifndef ROVERTCP_H
#define ROVERTCP_H

#ifdef EAGAIN
/* POSIX, SVR4 */
#define WouldBlock(E) (((E) == EAGAIN) || ((E) == EWOULDBLOCK))
#define InProgress(E) (((E) == EINPROGRESS) || ((E) == EAGAIN))
#else
/* BSD */
#define WouldBlock(E) ((E) == EWOULDBLOCK)
#define InProgress(E) ((E) == EINPROGRESS)
#endif /* EAGAIN */


typedef struct _URLinfo {
  char *url;
  char *protocol;
  char *hostname;
  int port;
  char *urlpath;
  char *base;
} URLinfo;

extern char *Simplify(char *filename);
extern char *parseURL(char **urlp, char **protocol, char **hostname,
		      int *port, char **urlpath, char *baseurl);
extern char *convertURL(URLinfo *uinfo);
extern char *newncopy(char *s, int n);

extern int RoverTCPOpenSocket(int port, int execClose);

/* Returns -1 if there is an error. Returns 0 if more lines are expected;
   otherwise, returns 1 if no more lines are expected. 
   If ptr is NULL, sets the state to nextState (nextState is -1 if this
   is an abort operation).
 */
typedef int (*NLFunction)(void *ClientData, char *ptr, void *nextState);
typedef void (*StateChangeFunc)(void *ClientData, void *newState);

typedef enum _NBR {
  GOT_BUFFER_OVERFLOW = -3,
  GOT_EOF = -2,
  GOT_ERROR = -1,
  GOT_NONL = 0,
  GOT_NL = 1
} NBR;

typedef enum _NBW {
  PUT_EOF = -2,
  PUT_ERROR = -1,
  PUT_CONTINUE = 0,
  PUT_DONE = 1
} NBW;

typedef struct _NBTcp {
  int sock;
  char *ptr;
  char *line;
  int offset;
  int size;
} NBTcp;


extern NBR readline(int fd, char *ptr, int *offset, int maxlen);
extern int readbuffer(int fd, char *ptr, int maxlen);
extern NBW writebuffer(int fd, char *ptr, int *offset, int *size);
extern int NLProcessLine(void *ClientData, NLFunction fcn, char *ptr, int size,
			 void *nextState);
extern int TCPRead(NBTcp *tcp, char *buf, int bufSize, int compress,
		   StateChangeFunc func, void *ClientData, void *nextState,
		   void *doneState, void *abortState);
extern int TCPGetLine(NBTcp *tcp, StateChangeFunc func, void *ClientData,
		      void *nextState, void *abortState);
extern int TCPGetBuffer(NBTcp *tcp, int *rc, StateChangeFunc func,
			void *ClientData, void *nextState, void *abortState);
extern int TCPPutBuffer(NBTcp *tcp, StateChangeFunc func,
			void *ClientData, void *nextState, void *abortState);

extern NBTcp *RoverTCPInit(int sock, int size);
extern void RoverTCPFree(NBTcp *tcp);
extern int RoverTCPAccept(int socket, int execClose);
extern int RoverSetCloseExec(int sockfd);
extern int RoverMakeSockNB(int sockfd, int connected);
extern void RoverGetRemoteInfo(int fd, char **remoteName, char **remoteIP);
extern int openTCP(char *host, int port);
extern int RoverTCPValidSocket(int socket);

#endif /* ROVERTCP_H */
