 /**********************************************************************
 *
 *
 *		    ROVER object execution engine
 *			  Anthony D. Joseph
 *		   Laboratory for Computer Science
 *		Massachusetts Institute of Technology
 *
 * Module: RoverTime.c
 *
 * Description: Common server and client time functions
 *
 *
 ***********************************************************************
 *   Copyright 1997
 *   Anthony D. Joseph
 *   Massachusetts Institute of Technology
 *
 *   Permission to use, copy, modify, and distribute this program
 *   for any purpose and without fee is hereby granted, provided
 *   that this copyright and permission notice appear on all copies
 *   and supporting documentation, the name of M.I.T. not be used
 *   in advertising or publicity pertaining to distribution of the
 *   program without specific prior permission, and notice be given
 *   in supporting documentation that copying and distribution is
 *   by permission of M.I.T.  M.I.T. makes no representations about
 *   the suitability of this software for any purpose.  It is pro-
 *   vided "as is" without express or implied warranty.		
 ***********************************************************************/

#include "RoverBase.h"
#include <time.h>

#define USE_LOCAL_COPY 304
#define HTTP_TIME_FORMAT "%a, %d %b %Y %T GMT" 

PRIVATE char *months[] = {
  "Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"
};


PRIVATE int Month2Num(char *month)
{
  int i;
  char **m = months;

  for (i=0 ; i < 12; i++, m++)
    if (!strncasecmp(*m,month,strlen(*m))) return(i);
  return(-1);
}

PUBLIC int TimeString2tm(char *time, struct tm *rv)
{
  char *t, mon[32], tmp[32];
  int year = 0, month = 0, day = 0, hour = 0, min = 0, sec = 0, rc;
  
  /* All formats start with the weekday, so skip to the first space */
  if(!(t = strchr(time, ' ')))
    return(0);
  else
    while (isspace(*t))
      ++t;
  
  if (isalpha(*t)) {		/* Format is ctime */
    rc = sscanf(t, "%31s %d %d:%d:%d %*s %d", mon, &day,&hour,&min,&sec,&year);
    if (rc != 6) return(0);
  } else if (t[2] == '-') {    /* Format is RFC 850 (normal HTTP) */
    rc = sscanf(t, "%31s %d:%d:%d", tmp, &hour, &min, &sec);
    if (rc != 4) return(0);
    tmp[2] = '\0';
    day = atoi(tmp);
    tmp[6] = '\0';
    strcpy(mname, &tmp[3]);
    year = atoi(&t[7]);
    if (year < 70) year += 100;	/* Prevent wraparound from ambiguity */
  } else {			/* Format is RFC 822 */
    rc = sscanf(t, "%d %31s %d %d:%d:%d", &day, mon, &year, &hour, &min, &sec);
    if (rc != 6) return(0);
  }
  month = find_month(mname);
  rv->tm_year = year;
  rv->tm_mon = month;
  rv->tm_mday = day;
  rv->tm_hour = hour;
  rv->tm_min = min;
  rv->tm_sec = sec;
  return(1);
}

PUBLIC int later_than(struct tm *lms, char *ims) {
  char *ip;
  char mname[MAX_STRING_LEN];
  int year = 0, month = 0, day = 0, hour = 0, min = 0, sec = 0, x;
  
  /* Whatever format we're looking at, it will start with weekday. */
  /* Skip to first space. */
  if(!(ip = strchr(ims,' ')))
    return 0;
  else
    while(isspace(*ip))
      ++ip;
  
  if(isalpha(*ip)) {
    /* ctime */
    sscanf(ip,"%s %d %d:%d:%d %*s %d",mname,&day,&hour,&min,&sec,&year);
  }
  else if(ip[2] == '-') {
    /* RFC 850 (normal HTTP) */
    char t[MAX_STRING_LEN];
    sscanf(ip,"%s %d:%d:%d",t,&hour,&min,&sec);
    t[2] = '\0';
    day = atoi(t);
    t[6] = '\0';
    strcpy(mname,&t[3]);
    x = atoi(&t[7]);
    /* Prevent wraparound from ambiguity */
    if(x < 70)
      x += 100;
    year = 1900 + x;
  }
  else {
    /* RFC 822 */
    sscanf(ip,"%d %s %d %d:%d:%d",&day,mname,&year,&hour,&min,&sec);
  }
  month = find_month(mname);
  
  if((x = (1900+lms->tm_year) - year))
    return x < 0;
  if((x = lms->tm_mon - month))
    return x < 0;
  if((x = lms->tm_mday - day))
    return x < 0;
  if((x = lms->tm_hour - hour))
    return x < 0;
  if((x = lms->tm_min - min))
    return x < 0;
  if((x = lms->tm_sec - sec))
    return x < 0;
  
  return 1;
}


int set_last_modified(time_t t, FILE *out) {
    struct tm *tms;
    char ts[MAX_STRING_LEN];

    tms = gmtime(&t);
    strftime(ts,MAX_STRING_LEN,HTTP_TIME_FORMAT,tms);
    strcpy(last_modified,ts);

    if(!ims[0])
        return 0;

    if(later_than(tms, ims))
        return die(USE_LOCAL_COPY,NULL,out);

    return 0;
}

