/**********************************************************************
 *
 *
 *		    ROVER object execution engine
 *			  Anthony D. Joseph
 *		   Laboratory for Computer Science
 *		Massachusetts Institute of Technology
 *
 * Module: HTApplication.h
 *
 * Description: Receives, forks, and executes a binary application
 *
 * Last Modified:  8-Feb-95 (adj)
 *
 * Global Functions:
 *    PUBLIC HTStream *HTAppRun(HTRequest *request,
 *				void *param,
 *				HTFormat input_format,
 *			        HTFormat output_format,
 *				HTStream *output_stream)
 *        Converter for application/x-application MIME stream. Downloads
 *        application binary code, creates pipes for IPC, and forks off
 *        a separate process.
 *
 * Global Variables: PUBLIC CONST HTStreamClass HTApplication
 *
 ***********************************************************************
 *   Copyright 1995
 *   Anthony D. Joseph
 *   Massachusetts Institute of Technology
 *
 *   Permission to use, copy, modify, and distribute this program
 *   for any purpose and without fee is hereby granted, provided
 *   that this copyright and permission notice appear on all copies
 *   and supporting documentation, the name of M.I.T. not be used
 *   in advertising or publicity pertaining to distribution of the
 *   program without specific prior permission, and notice be given
 *   in supporting documentation that copying and distribution is
 *   by permission of M.I.T.  M.I.T. makes no representations about
 *   the suitability of this software for any purpose.  It is pro-
 *   vided "as is" without express or implied warranty.		
 ***********************************************************************/

#ifndef HTRZ_H
#define HTRZ_H

#include "zlib.h"

/* Returns the number of bytes consumed. The function is responsible for
 *   moving the remaining data to the start of out
 */
typedef int (*writeFunction)(void *data, char *outStart,char *out,int newSize);

extern z_stream *InitUncompress();
extern int UncompressCore(z_stream *inf, char *in, int *inLen, char *out,
			  int *outLen, int outMax, int *written,
			  writeFunction fcn, void *data);
extern int Uncompress(z_stream *inf, FILE *sock, char *in, int *inLen,
		      char *out, int *outLen, int outMax, int *written);
extern int UncompressBuffer(z_stream *inf, char *in, int *inLen, char *out, 
			    int *outLen, int outMax, int *written);
extern int EndUncompress(z_stream *inf);

extern z_stream *InitCompress(int compressLevel);
extern int Compress(z_stream *c_stream, FILE *sock, char *in, int inLen,
		    char *out, int outLen, int *written);
extern int EndCompress(z_stream *c_stream, FILE *sock, char *out, int outLen,
		       int *written);
#endif /* HTRZ_H */
