/* html.h - brutally simple HTML parser for Rover Mosaic */
/* author: Alan deLespinasse */

/* Copyright 1995 Massachusetts Institute of Technology
   This code is not guaranteed to do anything in particular.
   Permission is granted to use, copy, modify, and distribute it
   as long as this notice remains intact. */

/* Instructions:

   Call HTMLnewparse to create a parser. The three arguments to the
   function are (1) a function to call on each stretch of plain text;
   (2) a function to call on each <>-enclosed HTML tag; and (3) a
   void*, which can be any extra data needed by these functions (i.e., 
   to identify which parse job the callback is for). Any of these
   values may be NULL. HTMLnewparse allocates space for the new structure
   and returns it.

   Call HTMLparse on each chunk of HTML source. It will call your callback
   functions at the appropriate times, passing them a pointer to the
   parser structure. The callback functions can look in the buf[] field
   for a zero-terminated string containing the appropriate bit of text,
   and the data field for the third parameter to the original
   HTMLnewparse call.

   Call the HTMLendparse when the entire HTML document has been passed
   through HTMLparse. This may call the callbacks again if necessary. The
   parser structure will automatically be deallocated.

   Note: The plain text callback will be called with the largest possible
   chunks of plain text, up to a fairly large predefined limit. This means
   that in many cases, especially between two matched HTML tags (i.e.,
   <TITLE> and </TITLE>), the function will only be called once between
   two successive tags.

*/

#ifndef rm_html_h
#define rm_html_h

#include <stdio.h>

typedef struct HTMLparser {
  void (*textcb)(struct HTMLparser*); /* callback for plain text */
  void (*tagcb)(struct HTMLparser*); /* callback for HTML tags */
  size_t length;		/* amount of text in buf */
  int intag,			/* 1 if currently inside <>'s */
  inquotes,			/* 1 if inside double quotes (") */
  incomment,			/* 1 if inside comment <-- --> */
  backslashed;			/* 1 if previous char was \ */
  void *data;			/* extra info for callbacks */
  char buf[1];			/* workspace */
} HTMLparser;

struct HTMLparser* HTMLnewparse(void (*textcb)(struct HTMLparser*),
				void (*tagcb)(struct HTMLparser*),void *);
void HTMLparse(struct HTMLparser*,char*,size_t);
void HTMLendparse(struct HTMLparser*);

#define BUFSIZE 32767	/* Size of read/write buffers */

#endif /* rm_html_h */
